﻿using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;

var serializationOptions = new JsonSerializerOptions()
{
    Converters =
    {
        new JsonStringEnumConverter()
    },
    NumberHandling = JsonNumberHandling.AllowReadingFromString,
};
string jsonString = File.ReadAllText("monotranscription.json");

TranscriptionJob transcriptionJob = JsonSerializer.Deserialize<TranscriptionJob>(jsonString, serializationOptions);

Dictionary<double, Line> dialogue = new Dictionary<double, Line>();

StringBuilder sb = new StringBuilder();
double speakerStartTime = 0;
bool lastItem = false;
string currentSpeaker = string.Empty;

foreach (var item in transcriptionJob.Results.Items)
{
    if (currentSpeaker == string.Empty)
    {
        speakerStartTime = item.StartTime;
        currentSpeaker = item.SpeakerLabel;
    }
    // change speaker, or last item in the list
    if (currentSpeaker != item.SpeakerLabel || item == transcriptionJob.Results.Items.Last())
    {
        if (item == transcriptionJob.Results.Items.Last())
        {
            AddItemToStringBuilder(sb, item);
            lastItem = true;
        }

        var line = new Line
        {
            Content = sb.ToString(),
            StartTime = speakerStartTime,
            SpeakerLabel = currentSpeaker
        };

        bool resultOfAdd = dialogue.TryAdd(speakerStartTime, line);
        if (!resultOfAdd)
        {
            Console.WriteLine($"Two lines with the same start time: {speakerStartTime}");
            // change the start time to avoid a duplicate key exception
            dialogue.TryAdd(speakerStartTime + 0.001, line);
        }

        if (!lastItem) // new speaker, not last item
        {
            speakerStartTime = item.StartTime;
            currentSpeaker = item.SpeakerLabel;
            sb.Clear();
            AddItemToStringBuilder(sb, item);
        }
    }
    else
    {
        AddItemToStringBuilder(sb, item);
    }
}

void AddItemToStringBuilder(StringBuilder sb, Item item)
{
    if (item.AudioType == AudioType.Punctuation)
    {
        sb.Remove(sb.Length - 1, 1); // remove space before punctuation
        sb.Append($"{item.Alternatives[0].Content} ");
    }
    else
    {
        sb.Append($"{item.Alternatives[0].Content} ");
    }
}


foreach (var keyValuePair in dialogue.OrderBy(x => x.Key))
{
    var timeSpan = TimeSpan.FromSeconds(keyValuePair.Key);
    Console.WriteLine($"{timeSpan.ToString(@"hh\:mm\:ss")}: {keyValuePair.Value.SpeakerLabel}: {keyValuePair.Value.Content}");
}
